import logging
from typing import List, Dict, Tuple, Union, Any
import threading

class DPage(object):
    def __init__(self):
        self.__logger: logging.Logger = None
        self.__int_mutex: threading.Lock = threading.Lock()
        self.__float_mutex: threading.Lock = threading.Lock()
        self.__str_mutex: threading.Lock = threading.Lock()
        self.__bool_mutex: threading.Lock = threading.Lock()
        self.__other_mutex: threading.Lock = threading.Lock()
        self.__int32_map: Dict[int, int] = dict()
        self.__int64_map: Dict[int, int] = dict()
        self.__float_map: Dict[int, float] = dict()
        self.__double_map: Dict[int, float] = dict()
        self.__string_map: Dict[int, str] = dict()
        self.__bool_map: Dict[int, bool] = dict()

        self.__save_map: Dict[int, Any] = {
            0: self.__save_fid_int32
            , 1: self.__save_fid_int64
            , 2: self.__save_fid_float
            , 3: self.__save_fid_double
            , 4: self.__save_fid_string
            , 5: self.__save_fid_other
            , 6: self.__save_fid_bool
        }
        self.__get_map: Dict[int, Any] = {
            0: self.__get_fid_int32
            , 1: self.__get_fid_int64
            , 2: self.__get_fid_float
            , 3: self.__get_fid_double
            , 4: self.__get_fid_string
            , 5: self.__get_fid_other
            , 6: self.__get_fid_bool
        }
        self.__bucket_size = 10000

    def init_component(self, logger: logging.Logger) -> None:
        self.__logger = logger

    def save_fid(self, fid_num: int, fid_value: Any) -> None:
        bucket_num = int((fid_num - 1) / self.__bucket_size)
        if bucket_num not in self.__save_map:
            return None
        self.__save_map[bucket_num](fid_num=fid_num, fid_value=fid_value)

    def get_fid(self, fid_num: int) -> Union[bool, Any]:
        bucket_num = int((fid_num - 1) / self.__bucket_size)
        if bucket_num not in self.__get_map:
            return (False, None)

        return (True, self.__get_map[bucket_num](fid_num=fid_num))

    def __save_fid_int32(self, fid_num: int, fid_value: int) -> None:
        with self.__int_mutex:
            self.__int32_map[fid_num] = fid_value

    def __save_fid_int64(self, fid_num: int, fid_value: int) -> None:
        with self.__int_mutex:
            self.__int64_map[fid_num] = fid_value

    def __save_fid_float(self, fid_num: int, fid_value: float) -> None:
        with self.__float_mutex:
            self.__float_map[fid_num] = fid_value

    def __save_fid_double(self, fid_num: int, fid_value: float) -> None:
        with self.__float_mutex:
            self.__double_map[fid_num] = fid_value

    def __save_fid_string(self, fid_num: int, fid_value: str) -> None:
        with self.__str_mutex:
            self.__string_map[fid_num] = fid_value

    def __save_fid_bool(self, fid_num: int, fid_value: float) -> None:
        with self.__bool_mutex:
            self.__bool_map[fid_num] = fid_value

    def __save_fid_other(self, fid_num: int, fid_value: Any) -> None:
        pass

    def __get_fid_int32(self, fid_num: int) -> Union[bool, int]:
        with self.__int_mutex:
            if fid_num in self.__int32_map:
                return (True, self.__int32_map[fid_num])
            else:
                return (False, None)

    def __get_fid_int64(self, fid_num: int) -> Union[bool, int]:
        with self.__int_mutex:
            if fid_num in self.__int64_map:
                return (True, self.__int64_map[fid_num])
            else:
                return (False, None)

    def __get_fid_float(self, fid_num: int) -> Union[bool, float]:
        with self.__float_mutex:
            if fid_num in self.__float_map:
                return (True, self.__float_map[fid_num])
            else:
                return (False, None)

    def __get_fid_double(self, fid_num: int) -> Union[bool, float]:
        with self.__float_mutex:
            if fid_num in self.__double_map:
                return (True, self.__double_map[fid_num])
            else:
                return (False, None)

    def __get_fid_string(self, fid_num: int) -> Union[bool, str]:
        with self.__str_mutex:
            if fid_num in self.__str_map:
                return (True, self.__str_map[fid_num])
            else:
                return (False, None)

    def __get_fid_bool(self, fid_num: int) -> Union[bool, bool]:
        with self.__bool_mutex:
            if fid_num in self.__bool_map:
                return (True, self.__bool_map[fid_num])
            else:
                return (False, None)

    def __get_fid_other(self, fid_num: int) -> Union[bool, Any]:
        pass



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ts_cop.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='ts_cop.proto',
  package='ts_cop',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0cts_cop.proto\x12\x06ts_cop\"\x13\n\x02\x43I\x12\r\n\x05value\x18\x01 \x01(\t\"%\n\x06\x43IList\x12\x1b\n\x07\x63i_list\x18\x01 \x03(\x0b\x32\n.ts_cop.CI\"\xbd\x01\n\x02SI\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\r\n\x05ratio\x18\x02 \x01(\x01\x12\x11\n\tparent_id\x18\x03 \x01(\t\x12\r\n\x05si_id\x18\x04 \x01(\t\x12\x0f\n\x07\x63reated\x18\x05 \x01(\x01\x12\x13\n\x0blast_update\x18\x06 \x01(\x01\x12\x15\n\rstrategy_name\x18\x07 \x01(\t\x12\x11\n\tlive_mode\x18\x08 \x01(\x08\x12\x13\n\x0bstrategy_id\x18\t \x01(\t\x12\x11\n\tsymbol_id\x18\n \x01(\t\"%\n\x06SIList\x12\x1b\n\x07si_list\x18\x01 \x03(\x0b\x32\n.ts_cop.SI\"\xa0\x03\n\x07TSOrder\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x10\n\x08quantity\x18\x02 \x01(\x01\x12\x15\n\rfill_quantity\x18\x03 \x01(\x01\x12\x11\n\tparent_id\x18\x04 \x01(\t\x12\x10\n\x08order_id\x18\x05 \x01(\t\x12\x19\n\x11platform_order_id\x18\x06 \x01(\t\x12\x0f\n\x07\x63reated\x18\x07 \x01(\x01\x12\x13\n\x0blast_update\x18\x08 \x01(\x01\x12\x15\n\rstrategy_name\x18\t \x01(\t\x12\x11\n\tlive_mode\x18\n \x01(\x08\x12\x13\n\x0bstrategy_id\x18\x0b \x01(\t\x12\x14\n\x0corder_status\x18\x0c \x01(\t\x12\x18\n\x10\x65xecution_broker\x18\r \x01(\t\x12\x17\n\x0f\x63learing_broker\x18\x0e \x01(\t\x12\x0f\n\x07\x61\x63\x63ount\x18\x0f \x01(\t\x12\r\n\x05price\x18\x10 \x01(\x01\x12\x0b\n\x03\x66\x65\x65\x18\x11 \x01(\x01\x12\x10\n\x08\x65xchange\x18\x12 \x01(\t\x12\x0b\n\x03\x63\x63y\x18\x13 \x01(\t\x12\x0f\n\x07\x66\x65\x65_ccy\x18\x14 \x01(\t\x12\x11\n\tsymbol_id\x18\x15 \x01(\t\"2\n\x0bTSOrderList\x12#\n\norder_list\x18\x01 \x03(\x0b\x32\x0f.ts_cop.TSOrder\"\x99\x03\n\x07TSTrade\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x10\n\x08quantity\x18\x02 \x01(\x01\x12\x11\n\tparent_id\x18\x03 \x01(\t\x12\x10\n\x08trade_id\x18\x04 \x01(\t\x12\x0f\n\x07\x63reated\x18\x05 \x01(\x01\x12\x13\n\x0blast_update\x18\x06 \x01(\x01\x12\x15\n\rstrategy_name\x18\x07 \x01(\t\x12\x11\n\tlive_mode\x18\x08 \x01(\x08\x12\x13\n\x0bstrategy_id\x18\t \x01(\t\x12\x14\n\x0ctrade_status\x18\n \x01(\t\x12\x18\n\x10\x65xecution_broker\x18\x0b \x01(\t\x12\x17\n\x0f\x63learing_broker\x18\x0c \x01(\t\x12\x17\n\x0fsettlement_date\x18\r \x01(\x01\x12\x10\n\x08\x66x_ratio\x18\x0e \x01(\x01\x12\x0f\n\x07\x61\x63\x63ount\x18\x0f \x01(\t\x12\r\n\x05price\x18\x10 \x01(\x01\x12\x0b\n\x03\x66\x65\x65\x18\x11 \x01(\x01\x12\x10\n\x08\x65xchange\x18\x12 \x01(\t\x12\x0b\n\x03\x63\x63y\x18\x13 \x01(\t\x12\x0f\n\x07\x66\x65\x65_ccy\x18\x14 \x01(\t\x12\x11\n\tsymbol_id\x18\x15 \x01(\t\"2\n\x0bTSTradeList\x12#\n\ntrade_list\x18\x01 \x03(\x0b\x32\x0f.ts_cop.TSTrade\"\xee\t\n\x03\x43op\x12\x0e\n\x06msg_id\x18\x01 \x01(\t\x12%\n\x08msg_type\x18\x02 \x01(\x0e\x32\x13.ts_cop.Cop.MsgType\x12\x0e\n\x06sender\x18\x03 \x01(\t\x12\x13\n\x0binstance_id\x18\x04 \x01(\x05\x12\x0f\n\x07seq_num\x18\x05 \x01(\x03\x12\x14\n\x0c\x63reated_time\x18\x06 \x01(\x01\x12,\n\tint32_map\x18\x07 \x03(\x0b\x32\x19.ts_cop.Cop.Int32MapEntry\x12,\n\tint64_map\x18\x08 \x03(\x0b\x32\x19.ts_cop.Cop.Int64MapEntry\x12,\n\tfloat_map\x18\t \x03(\x0b\x32\x19.ts_cop.Cop.FloatMapEntry\x12.\n\ndouble_map\x18\n \x03(\x0b\x32\x1a.ts_cop.Cop.DoubleMapEntry\x12.\n\nstring_map\x18\x0b \x03(\x0b\x32\x1a.ts_cop.Cop.StringMapEntry\x12*\n\x08\x62ool_map\x18\x0c \x03(\x0b\x32\x18.ts_cop.Cop.BoolMapEntry\x12,\n\torder_map\x18\r \x03(\x0b\x32\x19.ts_cop.Cop.OrderMapEntry\x12,\n\ttrade_map\x18\x0e \x03(\x0b\x32\x19.ts_cop.Cop.TradeMapEntry\x12&\n\x06si_map\x18\x0f \x03(\x0b\x32\x16.ts_cop.Cop.SiMapEntry\x12&\n\x06\x63i_map\x18\x10 \x03(\x0b\x32\x16.ts_cop.Cop.CiMapEntry\x1a/\n\rInt32MapEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1a/\n\rInt64MapEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\x03:\x02\x38\x01\x1a/\n\rFloatMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\x02:\x02\x38\x01\x1a\x30\n\x0e\x44oubleMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\x01:\x02\x38\x01\x1a\x30\n\x0eStringMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a.\n\x0c\x42oolMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a\x44\n\rOrderMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\"\n\x05value\x18\x02 \x01(\x0b\x32\x13.ts_cop.TSOrderList:\x02\x38\x01\x1a\x44\n\rTradeMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\"\n\x05value\x18\x02 \x01(\x0b\x32\x13.ts_cop.TSTradeList:\x02\x38\x01\x1a<\n\nSiMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\x1d\n\x05value\x18\x02 \x01(\x0b\x32\x0e.ts_cop.SIList:\x02\x38\x01\x1a<\n\nCiMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\x1d\n\x05value\x18\x02 \x01(\x0b\x32\x0e.ts_cop.CIList:\x02\x38\x01\"0\n\x07MsgType\x12\x0b\n\x07Listing\x10\x00\x12\x0c\n\x08Strategy\x10\x01\x12\n\n\x06System\x10\x02\"A\n\x06\x46idNum\x12\t\n\x05\x44ummy\x10\x00\x12\x08\n\x02\x43I\x10\xd1\x86\x03\x12\x08\n\x02SI\x10\xd2\x86\x03\x12\x0b\n\x05Order\x10\xd3\x86\x03\x12\x0b\n\x05Trade\x10\xd4\x86\x03\x62\x06proto3'
)



_COP_MSGTYPE = _descriptor.EnumDescriptor(
  name='MsgType',
  full_name='ts_cop.Cop.MsgType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='Listing', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Strategy', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='System', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2398,
  serialized_end=2446,
)
_sym_db.RegisterEnumDescriptor(_COP_MSGTYPE)

_COP_FIDNUM = _descriptor.EnumDescriptor(
  name='FidNum',
  full_name='ts_cop.Cop.FidNum',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='Dummy', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CI', index=1, number=50001,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SI', index=2, number=50002,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Order', index=3, number=50003,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Trade', index=4, number=50004,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2448,
  serialized_end=2513,
)
_sym_db.RegisterEnumDescriptor(_COP_FIDNUM)


_CI = _descriptor.Descriptor(
  name='CI',
  full_name='ts_cop.CI',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='ts_cop.CI.value', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=24,
  serialized_end=43,
)


_CILIST = _descriptor.Descriptor(
  name='CIList',
  full_name='ts_cop.CIList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='ci_list', full_name='ts_cop.CIList.ci_list', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=45,
  serialized_end=82,
)


_SI = _descriptor.Descriptor(
  name='SI',
  full_name='ts_cop.SI',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='symbol', full_name='ts_cop.SI.symbol', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ratio', full_name='ts_cop.SI.ratio', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='parent_id', full_name='ts_cop.SI.parent_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='si_id', full_name='ts_cop.SI.si_id', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='created', full_name='ts_cop.SI.created', index=4,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='last_update', full_name='ts_cop.SI.last_update', index=5,
      number=6, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='strategy_name', full_name='ts_cop.SI.strategy_name', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='live_mode', full_name='ts_cop.SI.live_mode', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='strategy_id', full_name='ts_cop.SI.strategy_id', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='symbol_id', full_name='ts_cop.SI.symbol_id', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=85,
  serialized_end=274,
)


_SILIST = _descriptor.Descriptor(
  name='SIList',
  full_name='ts_cop.SIList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='si_list', full_name='ts_cop.SIList.si_list', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=276,
  serialized_end=313,
)


_TSORDER = _descriptor.Descriptor(
  name='TSOrder',
  full_name='ts_cop.TSOrder',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='symbol', full_name='ts_cop.TSOrder.symbol', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='quantity', full_name='ts_cop.TSOrder.quantity', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fill_quantity', full_name='ts_cop.TSOrder.fill_quantity', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='parent_id', full_name='ts_cop.TSOrder.parent_id', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='order_id', full_name='ts_cop.TSOrder.order_id', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='platform_order_id', full_name='ts_cop.TSOrder.platform_order_id', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='created', full_name='ts_cop.TSOrder.created', index=6,
      number=7, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='last_update', full_name='ts_cop.TSOrder.last_update', index=7,
      number=8, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='strategy_name', full_name='ts_cop.TSOrder.strategy_name', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='live_mode', full_name='ts_cop.TSOrder.live_mode', index=9,
      number=10, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='strategy_id', full_name='ts_cop.TSOrder.strategy_id', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='order_status', full_name='ts_cop.TSOrder.order_status', index=11,
      number=12, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='execution_broker', full_name='ts_cop.TSOrder.execution_broker', index=12,
      number=13, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='clearing_broker', full_name='ts_cop.TSOrder.clearing_broker', index=13,
      number=14, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='account', full_name='ts_cop.TSOrder.account', index=14,
      number=15, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='price', full_name='ts_cop.TSOrder.price', index=15,
      number=16, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fee', full_name='ts_cop.TSOrder.fee', index=16,
      number=17, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='exchange', full_name='ts_cop.TSOrder.exchange', index=17,
      number=18, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ccy', full_name='ts_cop.TSOrder.ccy', index=18,
      number=19, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fee_ccy', full_name='ts_cop.TSOrder.fee_ccy', index=19,
      number=20, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='symbol_id', full_name='ts_cop.TSOrder.symbol_id', index=20,
      number=21, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=316,
  serialized_end=732,
)


_TSORDERLIST = _descriptor.Descriptor(
  name='TSOrderList',
  full_name='ts_cop.TSOrderList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='order_list', full_name='ts_cop.TSOrderList.order_list', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=734,
  serialized_end=784,
)


_TSTRADE = _descriptor.Descriptor(
  name='TSTrade',
  full_name='ts_cop.TSTrade',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='symbol', full_name='ts_cop.TSTrade.symbol', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='quantity', full_name='ts_cop.TSTrade.quantity', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='parent_id', full_name='ts_cop.TSTrade.parent_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='trade_id', full_name='ts_cop.TSTrade.trade_id', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='created', full_name='ts_cop.TSTrade.created', index=4,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='last_update', full_name='ts_cop.TSTrade.last_update', index=5,
      number=6, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='strategy_name', full_name='ts_cop.TSTrade.strategy_name', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='live_mode', full_name='ts_cop.TSTrade.live_mode', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='strategy_id', full_name='ts_cop.TSTrade.strategy_id', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='trade_status', full_name='ts_cop.TSTrade.trade_status', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='execution_broker', full_name='ts_cop.TSTrade.execution_broker', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='clearing_broker', full_name='ts_cop.TSTrade.clearing_broker', index=11,
      number=12, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='settlement_date', full_name='ts_cop.TSTrade.settlement_date', index=12,
      number=13, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fx_ratio', full_name='ts_cop.TSTrade.fx_ratio', index=13,
      number=14, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='account', full_name='ts_cop.TSTrade.account', index=14,
      number=15, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='price', full_name='ts_cop.TSTrade.price', index=15,
      number=16, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fee', full_name='ts_cop.TSTrade.fee', index=16,
      number=17, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='exchange', full_name='ts_cop.TSTrade.exchange', index=17,
      number=18, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ccy', full_name='ts_cop.TSTrade.ccy', index=18,
      number=19, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fee_ccy', full_name='ts_cop.TSTrade.fee_ccy', index=19,
      number=20, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='symbol_id', full_name='ts_cop.TSTrade.symbol_id', index=20,
      number=21, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=787,
  serialized_end=1196,
)


_TSTRADELIST = _descriptor.Descriptor(
  name='TSTradeList',
  full_name='ts_cop.TSTradeList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='trade_list', full_name='ts_cop.TSTradeList.trade_list', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1198,
  serialized_end=1248,
)


_COP_INT32MAPENTRY = _descriptor.Descriptor(
  name='Int32MapEntry',
  full_name='ts_cop.Cop.Int32MapEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ts_cop.Cop.Int32MapEntry.key', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ts_cop.Cop.Int32MapEntry.value', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1839,
  serialized_end=1886,
)

_COP_INT64MAPENTRY = _descriptor.Descriptor(
  name='Int64MapEntry',
  full_name='ts_cop.Cop.Int64MapEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ts_cop.Cop.Int64MapEntry.key', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ts_cop.Cop.Int64MapEntry.value', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1888,
  serialized_end=1935,
)

_COP_FLOATMAPENTRY = _descriptor.Descriptor(
  name='FloatMapEntry',
  full_name='ts_cop.Cop.FloatMapEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ts_cop.Cop.FloatMapEntry.key', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ts_cop.Cop.FloatMapEntry.value', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1937,
  serialized_end=1984,
)

_COP_DOUBLEMAPENTRY = _descriptor.Descriptor(
  name='DoubleMapEntry',
  full_name='ts_cop.Cop.DoubleMapEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ts_cop.Cop.DoubleMapEntry.key', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ts_cop.Cop.DoubleMapEntry.value', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1986,
  serialized_end=2034,
)

_COP_STRINGMAPENTRY = _descriptor.Descriptor(
  name='StringMapEntry',
  full_name='ts_cop.Cop.StringMapEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ts_cop.Cop.StringMapEntry.key', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ts_cop.Cop.StringMapEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2036,
  serialized_end=2084,
)

_COP_BOOLMAPENTRY = _descriptor.Descriptor(
  name='BoolMapEntry',
  full_name='ts_cop.Cop.BoolMapEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ts_cop.Cop.BoolMapEntry.key', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ts_cop.Cop.BoolMapEntry.value', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2086,
  serialized_end=2132,
)

_COP_ORDERMAPENTRY = _descriptor.Descriptor(
  name='OrderMapEntry',
  full_name='ts_cop.Cop.OrderMapEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ts_cop.Cop.OrderMapEntry.key', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ts_cop.Cop.OrderMapEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2134,
  serialized_end=2202,
)

_COP_TRADEMAPENTRY = _descriptor.Descriptor(
  name='TradeMapEntry',
  full_name='ts_cop.Cop.TradeMapEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ts_cop.Cop.TradeMapEntry.key', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ts_cop.Cop.TradeMapEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2204,
  serialized_end=2272,
)

_COP_SIMAPENTRY = _descriptor.Descriptor(
  name='SiMapEntry',
  full_name='ts_cop.Cop.SiMapEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ts_cop.Cop.SiMapEntry.key', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ts_cop.Cop.SiMapEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2274,
  serialized_end=2334,
)

_COP_CIMAPENTRY = _descriptor.Descriptor(
  name='CiMapEntry',
  full_name='ts_cop.Cop.CiMapEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ts_cop.Cop.CiMapEntry.key', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ts_cop.Cop.CiMapEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2336,
  serialized_end=2396,
)

_COP = _descriptor.Descriptor(
  name='Cop',
  full_name='ts_cop.Cop',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='msg_id', full_name='ts_cop.Cop.msg_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='msg_type', full_name='ts_cop.Cop.msg_type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sender', full_name='ts_cop.Cop.sender', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='instance_id', full_name='ts_cop.Cop.instance_id', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='seq_num', full_name='ts_cop.Cop.seq_num', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='created_time', full_name='ts_cop.Cop.created_time', index=5,
      number=6, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='int32_map', full_name='ts_cop.Cop.int32_map', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='int64_map', full_name='ts_cop.Cop.int64_map', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='float_map', full_name='ts_cop.Cop.float_map', index=8,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='double_map', full_name='ts_cop.Cop.double_map', index=9,
      number=10, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='string_map', full_name='ts_cop.Cop.string_map', index=10,
      number=11, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bool_map', full_name='ts_cop.Cop.bool_map', index=11,
      number=12, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='order_map', full_name='ts_cop.Cop.order_map', index=12,
      number=13, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='trade_map', full_name='ts_cop.Cop.trade_map', index=13,
      number=14, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='si_map', full_name='ts_cop.Cop.si_map', index=14,
      number=15, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ci_map', full_name='ts_cop.Cop.ci_map', index=15,
      number=16, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_COP_INT32MAPENTRY, _COP_INT64MAPENTRY, _COP_FLOATMAPENTRY, _COP_DOUBLEMAPENTRY, _COP_STRINGMAPENTRY, _COP_BOOLMAPENTRY, _COP_ORDERMAPENTRY, _COP_TRADEMAPENTRY, _COP_SIMAPENTRY, _COP_CIMAPENTRY, ],
  enum_types=[
    _COP_MSGTYPE,
    _COP_FIDNUM,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1251,
  serialized_end=2513,
)

_CILIST.fields_by_name['ci_list'].message_type = _CI
_SILIST.fields_by_name['si_list'].message_type = _SI
_TSORDERLIST.fields_by_name['order_list'].message_type = _TSORDER
_TSTRADELIST.fields_by_name['trade_list'].message_type = _TSTRADE
_COP_INT32MAPENTRY.containing_type = _COP
_COP_INT64MAPENTRY.containing_type = _COP
_COP_FLOATMAPENTRY.containing_type = _COP
_COP_DOUBLEMAPENTRY.containing_type = _COP
_COP_STRINGMAPENTRY.containing_type = _COP
_COP_BOOLMAPENTRY.containing_type = _COP
_COP_ORDERMAPENTRY.fields_by_name['value'].message_type = _TSORDERLIST
_COP_ORDERMAPENTRY.containing_type = _COP
_COP_TRADEMAPENTRY.fields_by_name['value'].message_type = _TSTRADELIST
_COP_TRADEMAPENTRY.containing_type = _COP
_COP_SIMAPENTRY.fields_by_name['value'].message_type = _SILIST
_COP_SIMAPENTRY.containing_type = _COP
_COP_CIMAPENTRY.fields_by_name['value'].message_type = _CILIST
_COP_CIMAPENTRY.containing_type = _COP
_COP.fields_by_name['msg_type'].enum_type = _COP_MSGTYPE
_COP.fields_by_name['int32_map'].message_type = _COP_INT32MAPENTRY
_COP.fields_by_name['int64_map'].message_type = _COP_INT64MAPENTRY
_COP.fields_by_name['float_map'].message_type = _COP_FLOATMAPENTRY
_COP.fields_by_name['double_map'].message_type = _COP_DOUBLEMAPENTRY
_COP.fields_by_name['string_map'].message_type = _COP_STRINGMAPENTRY
_COP.fields_by_name['bool_map'].message_type = _COP_BOOLMAPENTRY
_COP.fields_by_name['order_map'].message_type = _COP_ORDERMAPENTRY
_COP.fields_by_name['trade_map'].message_type = _COP_TRADEMAPENTRY
_COP.fields_by_name['si_map'].message_type = _COP_SIMAPENTRY
_COP.fields_by_name['ci_map'].message_type = _COP_CIMAPENTRY
_COP_MSGTYPE.containing_type = _COP
_COP_FIDNUM.containing_type = _COP
DESCRIPTOR.message_types_by_name['CI'] = _CI
DESCRIPTOR.message_types_by_name['CIList'] = _CILIST
DESCRIPTOR.message_types_by_name['SI'] = _SI
DESCRIPTOR.message_types_by_name['SIList'] = _SILIST
DESCRIPTOR.message_types_by_name['TSOrder'] = _TSORDER
DESCRIPTOR.message_types_by_name['TSOrderList'] = _TSORDERLIST
DESCRIPTOR.message_types_by_name['TSTrade'] = _TSTRADE
DESCRIPTOR.message_types_by_name['TSTradeList'] = _TSTRADELIST
DESCRIPTOR.message_types_by_name['Cop'] = _COP
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

CI = _reflection.GeneratedProtocolMessageType('CI', (_message.Message,), {
  'DESCRIPTOR' : _CI,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.CI)
  })
_sym_db.RegisterMessage(CI)

CIList = _reflection.GeneratedProtocolMessageType('CIList', (_message.Message,), {
  'DESCRIPTOR' : _CILIST,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.CIList)
  })
_sym_db.RegisterMessage(CIList)

SI = _reflection.GeneratedProtocolMessageType('SI', (_message.Message,), {
  'DESCRIPTOR' : _SI,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.SI)
  })
_sym_db.RegisterMessage(SI)

SIList = _reflection.GeneratedProtocolMessageType('SIList', (_message.Message,), {
  'DESCRIPTOR' : _SILIST,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.SIList)
  })
_sym_db.RegisterMessage(SIList)

TSOrder = _reflection.GeneratedProtocolMessageType('TSOrder', (_message.Message,), {
  'DESCRIPTOR' : _TSORDER,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.TSOrder)
  })
_sym_db.RegisterMessage(TSOrder)

TSOrderList = _reflection.GeneratedProtocolMessageType('TSOrderList', (_message.Message,), {
  'DESCRIPTOR' : _TSORDERLIST,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.TSOrderList)
  })
_sym_db.RegisterMessage(TSOrderList)

TSTrade = _reflection.GeneratedProtocolMessageType('TSTrade', (_message.Message,), {
  'DESCRIPTOR' : _TSTRADE,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.TSTrade)
  })
_sym_db.RegisterMessage(TSTrade)

TSTradeList = _reflection.GeneratedProtocolMessageType('TSTradeList', (_message.Message,), {
  'DESCRIPTOR' : _TSTRADELIST,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.TSTradeList)
  })
_sym_db.RegisterMessage(TSTradeList)

Cop = _reflection.GeneratedProtocolMessageType('Cop', (_message.Message,), {

  'Int32MapEntry' : _reflection.GeneratedProtocolMessageType('Int32MapEntry', (_message.Message,), {
    'DESCRIPTOR' : _COP_INT32MAPENTRY,
    '__module__' : 'ts_cop_pb2'
    # @@protoc_insertion_point(class_scope:ts_cop.Cop.Int32MapEntry)
    })
  ,

  'Int64MapEntry' : _reflection.GeneratedProtocolMessageType('Int64MapEntry', (_message.Message,), {
    'DESCRIPTOR' : _COP_INT64MAPENTRY,
    '__module__' : 'ts_cop_pb2'
    # @@protoc_insertion_point(class_scope:ts_cop.Cop.Int64MapEntry)
    })
  ,

  'FloatMapEntry' : _reflection.GeneratedProtocolMessageType('FloatMapEntry', (_message.Message,), {
    'DESCRIPTOR' : _COP_FLOATMAPENTRY,
    '__module__' : 'ts_cop_pb2'
    # @@protoc_insertion_point(class_scope:ts_cop.Cop.FloatMapEntry)
    })
  ,

  'DoubleMapEntry' : _reflection.GeneratedProtocolMessageType('DoubleMapEntry', (_message.Message,), {
    'DESCRIPTOR' : _COP_DOUBLEMAPENTRY,
    '__module__' : 'ts_cop_pb2'
    # @@protoc_insertion_point(class_scope:ts_cop.Cop.DoubleMapEntry)
    })
  ,

  'StringMapEntry' : _reflection.GeneratedProtocolMessageType('StringMapEntry', (_message.Message,), {
    'DESCRIPTOR' : _COP_STRINGMAPENTRY,
    '__module__' : 'ts_cop_pb2'
    # @@protoc_insertion_point(class_scope:ts_cop.Cop.StringMapEntry)
    })
  ,

  'BoolMapEntry' : _reflection.GeneratedProtocolMessageType('BoolMapEntry', (_message.Message,), {
    'DESCRIPTOR' : _COP_BOOLMAPENTRY,
    '__module__' : 'ts_cop_pb2'
    # @@protoc_insertion_point(class_scope:ts_cop.Cop.BoolMapEntry)
    })
  ,

  'OrderMapEntry' : _reflection.GeneratedProtocolMessageType('OrderMapEntry', (_message.Message,), {
    'DESCRIPTOR' : _COP_ORDERMAPENTRY,
    '__module__' : 'ts_cop_pb2'
    # @@protoc_insertion_point(class_scope:ts_cop.Cop.OrderMapEntry)
    })
  ,

  'TradeMapEntry' : _reflection.GeneratedProtocolMessageType('TradeMapEntry', (_message.Message,), {
    'DESCRIPTOR' : _COP_TRADEMAPENTRY,
    '__module__' : 'ts_cop_pb2'
    # @@protoc_insertion_point(class_scope:ts_cop.Cop.TradeMapEntry)
    })
  ,

  'SiMapEntry' : _reflection.GeneratedProtocolMessageType('SiMapEntry', (_message.Message,), {
    'DESCRIPTOR' : _COP_SIMAPENTRY,
    '__module__' : 'ts_cop_pb2'
    # @@protoc_insertion_point(class_scope:ts_cop.Cop.SiMapEntry)
    })
  ,

  'CiMapEntry' : _reflection.GeneratedProtocolMessageType('CiMapEntry', (_message.Message,), {
    'DESCRIPTOR' : _COP_CIMAPENTRY,
    '__module__' : 'ts_cop_pb2'
    # @@protoc_insertion_point(class_scope:ts_cop.Cop.CiMapEntry)
    })
  ,
  'DESCRIPTOR' : _COP,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.Cop)
  })
_sym_db.RegisterMessage(Cop)
_sym_db.RegisterMessage(Cop.Int32MapEntry)
_sym_db.RegisterMessage(Cop.Int64MapEntry)
_sym_db.RegisterMessage(Cop.FloatMapEntry)
_sym_db.RegisterMessage(Cop.DoubleMapEntry)
_sym_db.RegisterMessage(Cop.StringMapEntry)
_sym_db.RegisterMessage(Cop.BoolMapEntry)
_sym_db.RegisterMessage(Cop.OrderMapEntry)
_sym_db.RegisterMessage(Cop.TradeMapEntry)
_sym_db.RegisterMessage(Cop.SiMapEntry)
_sym_db.RegisterMessage(Cop.CiMapEntry)


_COP_INT32MAPENTRY._options = None
_COP_INT64MAPENTRY._options = None
_COP_FLOATMAPENTRY._options = None
_COP_DOUBLEMAPENTRY._options = None
_COP_STRINGMAPENTRY._options = None
_COP_BOOLMAPENTRY._options = None
_COP_ORDERMAPENTRY._options = None
_COP_TRADEMAPENTRY._options = None
_COP_SIMAPENTRY._options = None
_COP_CIMAPENTRY._options = None
# @@protoc_insertion_point(module_scope)



import logging
import os
import time

import sfdevtools.observability.log_helper as lh
import sfdevtools.data_control.TSDataCtrl as TSDataCtrl
import sfdevtools.grpc_protos.ts_cop_pb2 as ts_cop_pb2
import sfdevtools.grpc_protos.ts_cop_pb2_grpc as ts_cop_pb2_grpc

def main():
    logger: logging.Logger = lh.init_logger(logger_name="TSDataCtrl_pub_logger", is_json_output=False)
    env_v = {"proxy_host": os.getenv("P_HOST", default="0.0.0.0")
             , "proxy_in_port": os.getenv("P_IN_PORT", default="5557")
             , "proxy_out_port": os.getenv("P_OUT_PORT", default="5558")}
    logger.info(env_v)

    data_ctrl: TSDataCtrl = TSDataCtrl.TSDataCtrl()
    data_ctrl.init_component(logger=logger)
    data_ctrl.init_publish(pub_host=env_v["proxy_host"]
                           , pub_port=env_v["proxy_in_port"]
                           , sender_name="RSA_True"
                           , instance_id=1)

    num_count = 0
    while True:
        cop = ts_cop_pb2.Cop()
        cop.int32_map[1] = num_count
        num_count += 1
        logger.info(f"send cop: {cop}")
        data_ctrl.send_cop(topic="RSA_True", cop=cop)
        time.sleep(1)

if __name__ == '__main__':
    main()



class StrategyInsight(object):
    def __init__(self, symbol: str, ratio: float):
        self.symbol: str = symbol
        self.symbol_id: str = ""
        self.ratio: float = ratio
        self.parent_id: str = ""
        self.si_id: str = ""
        self.created: datetime = None
        self.last_update: datetime = None
        self.strategy_name: str = ""
        self.live_mode: bool = False
        self.strategy_id: str = ""

    def __eq__(self, other) -> bool:
        if isinstance(other, StrategyInsight):
            return self.symbol == other.symbol\
                and self.ratio == other.ratio
        else:
            return False

    def __str__(self):
        return f"{self.symbol},{self.ratio}"

class TS_Order(object):
    def __init__(self):
        self.symbol: str = ""
        self.symbol_id: str = ""
        self.quantity: float = 0
        self.fill_quantity: float = 0
        self.parent_id: str = ""
        self.order_id: str = ""
        self.platform_order_id: int = 0
        self.created: datetime = None
        self.last_update: datetime = None
        self.strategy_name: str = ""
        self.live_mode: bool = False
        self.strategy_id: str = ""
        # https://github.com/QuantConnect/Lean/blob/master/Common/Orders/OrderTypes.cs#L107
        self.order_status: str = ""
        self.execution_broker: str = ""
        self.clearing_broker: str = ""
        self.account: str = ""
        self.price: float = 0
        self.fee: float = 0
        self.fee_ccy: str = ""
        self.exchange: str = ""
        self.ccy: str = ""

    def __eq__(self, other) -> bool:
        if isinstance(other, TS_Order):
            return self.order_id == other.order_id\
                and self.platform_order_id == other.platform_order_id
        else:
            return False

    def __str__(self):
        return f"{self.symbol},{self.quantity}"

class TS_Trade(object):
    def __init__(self):
        self.symbol: str = ""
        self.symbol_id: str = ""
        self.quantity: float = 0
        self.parent_id: str = ""
        self.trade_id: str = ""
        self.created: datetime = None
        self.last_update: datetime = None
        self.strategy_name: str = ""
        self.live_mode: bool = False
        self.strategy_id: str = ""
        # https://github.com/QuantConnect/Lean/blob/master/Common/Orders/OrderTypes.cs#L107
        self.trade_status: str = ""
        self.execution_broker: str = ""
        self.clearing_broker: str = ""
        self.settlement_date: datetime = None
        self.fx_ratio: float = 0
        self.account: str = ""
        self.price: float = 0
        self.fee: float = 0
        self.fee_ccy: str = ""
        self.exchange: str = ""
        self.ccy: str = ""

    def __eq__(self, other) -> bool:
        if isinstance(other, TS_Order):
            return self.order_id == other.order_id\
                and self.platform_order_id == other.platform_order_id
        else:
            return False

    def __str__(self):
        return f"{self.symbol},{self.quantity}"



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import sfdevtools.grpc_protos.tweety_pb2 as tweety__pb2


class TweetyStub(object):
    """The greeting service definition.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetStrategyConfigId = channel.unary_unary(
                '/tweety.Tweety/GetStrategyConfigId',
                request_serializer=tweety__pb2.GetStrategyConfigId_Msg.SerializeToString,
                response_deserializer=tweety__pb2.GetStrategyConfigId_Reply.FromString,
                )
        self.SaveStrategyConfig = channel.unary_unary(
                '/tweety.Tweety/SaveStrategyConfig',
                request_serializer=tweety__pb2.SaveStrategyConfig_Msg.SerializeToString,
                response_deserializer=tweety__pb2.SaveStrategyConfig_Reply.FromString,
                )
        self.HealthCheck = channel.unary_unary(
                '/tweety.Tweety/HealthCheck',
                request_serializer=tweety__pb2.Ping.SerializeToString,
                response_deserializer=tweety__pb2.Pong.FromString,
                )


class TweetyServicer(object):
    """The greeting service definition.
    """

    def GetStrategyConfigId(self, request, context):
        """Unary - request and respond
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SaveStrategyConfig(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def HealthCheck(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_TweetyServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetStrategyConfigId': grpc.unary_unary_rpc_method_handler(
                    servicer.GetStrategyConfigId,
                    request_deserializer=tweety__pb2.GetStrategyConfigId_Msg.FromString,
                    response_serializer=tweety__pb2.GetStrategyConfigId_Reply.SerializeToString,
            ),
            'SaveStrategyConfig': grpc.unary_unary_rpc_method_handler(
                    servicer.SaveStrategyConfig,
                    request_deserializer=tweety__pb2.SaveStrategyConfig_Msg.FromString,
                    response_serializer=tweety__pb2.SaveStrategyConfig_Reply.SerializeToString,
            ),
            'HealthCheck': grpc.unary_unary_rpc_method_handler(
                    servicer.HealthCheck,
                    request_deserializer=tweety__pb2.Ping.FromString,
                    response_serializer=tweety__pb2.Pong.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'tweety.Tweety', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Tweety(object):
    """The greeting service definition.
    """

    @staticmethod
    def GetStrategyConfigId(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/tweety.Tweety/GetStrategyConfigId',
            tweety__pb2.GetStrategyConfigId_Msg.SerializeToString,
            tweety__pb2.GetStrategyConfigId_Reply.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SaveStrategyConfig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/tweety.Tweety/SaveStrategyConfig',
            tweety__pb2.SaveStrategyConfig_Msg.SerializeToString,
            tweety__pb2.SaveStrategyConfig_Reply.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def HealthCheck(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/tweety.Tweety/HealthCheck',
            tweety__pb2.Ping.SerializeToString,
            tweety__pb2.Pong.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: magpie.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='magpie.proto',
  package='magpie',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0cmagpie.proto\x12\x06magpie\"\x17\n\x04Ping\x12\x0f\n\x07message\x18\x01 \x01(\t\"\x17\n\x04Pong\x12\x0f\n\x07message\x18\x01 \x01(\t\"\'\n\x17UpdateRefDataFromQC_Msg\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\t\"\x1a\n\tNoneReply\x12\r\n\x05reply\x18\x01 \x01(\t2~\n\x06Magpie\x12)\n\x0bHealthCheck\x12\x0c.magpie.Ping\x1a\x0c.magpie.Pong\x12I\n\x13UpdateRefDataFromQC\x12\x1f.magpie.UpdateRefDataFromQC_Msg\x1a\x11.magpie.NoneReplyb\x06proto3'
)




_PING = _descriptor.Descriptor(
  name='Ping',
  full_name='magpie.Ping',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='magpie.Ping.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=24,
  serialized_end=47,
)


_PONG = _descriptor.Descriptor(
  name='Pong',
  full_name='magpie.Pong',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='magpie.Pong.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=49,
  serialized_end=72,
)


_UPDATEREFDATAFROMQC_MSG = _descriptor.Descriptor(
  name='UpdateRefDataFromQC_Msg',
  full_name='magpie.UpdateRefDataFromQC_Msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='magpie.UpdateRefDataFromQC_Msg.data', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=74,
  serialized_end=113,
)


_NONEREPLY = _descriptor.Descriptor(
  name='NoneReply',
  full_name='magpie.NoneReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='reply', full_name='magpie.NoneReply.reply', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=115,
  serialized_end=141,
)

DESCRIPTOR.message_types_by_name['Ping'] = _PING
DESCRIPTOR.message_types_by_name['Pong'] = _PONG
DESCRIPTOR.message_types_by_name['UpdateRefDataFromQC_Msg'] = _UPDATEREFDATAFROMQC_MSG
DESCRIPTOR.message_types_by_name['NoneReply'] = _NONEREPLY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Ping = _reflection.GeneratedProtocolMessageType('Ping', (_message.Message,), {
  'DESCRIPTOR' : _PING,
  '__module__' : 'magpie_pb2'
  # @@protoc_insertion_point(class_scope:magpie.Ping)
  })
_sym_db.RegisterMessage(Ping)

Pong = _reflection.GeneratedProtocolMessageType('Pong', (_message.Message,), {
  'DESCRIPTOR' : _PONG,
  '__module__' : 'magpie_pb2'
  # @@protoc_insertion_point(class_scope:magpie.Pong)
  })
_sym_db.RegisterMessage(Pong)

UpdateRefDataFromQC_Msg = _reflection.GeneratedProtocolMessageType('UpdateRefDataFromQC_Msg', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEREFDATAFROMQC_MSG,
  '__module__' : 'magpie_pb2'
  # @@protoc_insertion_point(class_scope:magpie.UpdateRefDataFromQC_Msg)
  })
_sym_db.RegisterMessage(UpdateRefDataFromQC_Msg)

NoneReply = _reflection.GeneratedProtocolMessageType('NoneReply', (_message.Message,), {
  'DESCRIPTOR' : _NONEREPLY,
  '__module__' : 'magpie_pb2'
  # @@protoc_insertion_point(class_scope:magpie.NoneReply)
  })
_sym_db.RegisterMessage(NoneReply)



_MAGPIE = _descriptor.ServiceDescriptor(
  name='Magpie',
  full_name='magpie.Magpie',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=143,
  serialized_end=269,
  methods=[
  _descriptor.MethodDescriptor(
    name='HealthCheck',
    full_name='magpie.Magpie.HealthCheck',
    index=0,
    containing_service=None,
    input_type=_PING,
    output_type=_PONG,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateRefDataFromQC',
    full_name='magpie.Magpie.UpdateRefDataFromQC',
    index=1,
    containing_service=None,
    input_type=_UPDATEREFDATAFROMQC_MSG,
    output_type=_NONEREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_MAGPIE)

DESCRIPTOR.services_by_name['Magpie'] = _MAGPIE

# @@protoc_insertion_point(module_scope)



import logging
import threading
from typing import List, Dict, Tuple, Union, Any
import copy

from sfdevtools.data_cache.DComponents import StrategyInsight, TS_Order, TS_Trade
import sfdevtools.grpc_protos.ts_cop_pb2 as ts_cop_pb2
import sfdevtools.grpc_protos.ts_cop_pb2_grpc as ts_cop_pb2_grpc
import sfdevtools.devTools.TimelyCache as TimelyCache

class DStrategy(object):
    def __init__(self):
        self.__logger: logging.Logger = None
        self.__max_hist_orders: int = 1000
        self.__max_hist_trades: int = 1000
        self.__max_hist_si: int = 1000
        self.__si_dict: Dict[str, List[StrategyInsight]] = dict() # key: si_id
        self.__si_list: List[str] = list() # List of si_id
        self.__ci: Dict[str, Any] = dict()
        self.__ci_list: List[Dict[str, Any]] = list() # calculation insight
        self.__ci_id: str = ""
        self.__order_cache: TimelyCache.TimelyCache_DupKey = TimelyCache.TimelyCache_DupKey() # key: platform_order_id --- keep latest snapshot
        self.__trade_cache: TimelyCache.TimelyCache_UniKey = TimelyCache.TimelyCache_UniKey() # key: trade_id --- keep latest records in hash
        self.__mutex: threading.Lock = threading.Lock()

        self.__int_mutex: threading.Lock = threading.Lock()
        self.__float_mutex: threading.Lock = threading.Lock()
        self.__str_mutex: threading.Lock = threading.Lock()
        self.__bool_mutex: threading.Lock = threading.Lock()
        self.__other_mutex: threading.Lock = threading.Lock()
        self.__int32_map: Dict[int, int] = dict()
        self.__int64_map: Dict[int, int] = dict()
        self.__float_map: Dict[int, float] = dict()
        self.__double_map: Dict[int, float] = dict()
        self.__string_map: Dict[int, str] = dict()
        self.__bool_map: Dict[int, bool] = dict()

        self.__save_map: Dict[int, Any] = {
            0: self.__save_fid_int32
            , 1: self.__save_fid_int64
            , 2: self.__save_fid_float
            , 3: self.__save_fid_double
            , 4: self.__save_fid_string
            , 5: self.__save_fid_other
            , 6: self.__save_fid_bool
        }
        self.__get_map: Dict[int, Any] = {
            0: self.__get_fid_int32
            , 1: self.__get_fid_int64
            , 2: self.__get_fid_float
            , 3: self.__get_fid_double
            , 4: self.__get_fid_string
            , 5: self.__get_fid_other
            , 6: self.__get_fid_bool
        }
        self.__save_other_map: Dict[int, Any] = {
            ts_cop_pb2.Cop.FidNum.CI: self.save_ci
            , ts_cop_pb2.Cop.FidNum.SI: self.save_si
            , ts_cop_pb2.Cop.FidNum.Order: self.save_orders
            , ts_cop_pb2.Cop.FidNum.Trade: self.save_trades
        }
        self.__get_other_map: Dict[int, Any] = {
            ts_cop_pb2.Cop.FidNum.CI: self.get_ci
            , ts_cop_pb2.Cop.FidNum.SI: self.get_si
            , ts_cop_pb2.Cop.FidNum.Order: self.get_orders
            , ts_cop_pb2.Cop.FidNum.Trade: self.get_trades
        }
        self.__bucket_size = 10000

    def init_component(self
                       , logger: logging.Logger
                       , max_hist_orders: int = 1000
                       , max_hist_trades: int = 1000
                       , max_hist_si: int = 1000) -> None:
        self.__logger = logger
        self.__max_hist_orders = max_hist_orders
        self.__max_hist_trades = max_hist_trades
        self.__max_hist_si = max_hist_si

    def get_strategy_insight(self) -> List[StrategyInsight]:
        with self.__mutex:
            return self.__si_list

    def save_si(self, si: List[StrategyInsight]) -> bool:
        with self.__mutex:
            if len(si) == 0:
                return True

            self.__si_list.append(si[0].si_id)
            self.__si_dict[si[0].si_id] = si

            # remove old records
            if len(self.__si_list) > self.__max_hist_si:
                extra_records = len(self.__si_list) - self.__max_hist_si
                # remove records from dictionary
                for old_si_id in self.__si_list[:extra_records]:
                    if old_si_id in self.__si_dict:
                        del self.__si_dict[old_si_id]

                # remove records from list
                del self.__si_list[:extra_records]

            return True
        return False

    def is_latest_si(self, si: List[StrategyInsight]) -> bool:
        with self.__mutex:
            return self.__is_same_si(si1=self.__si_list, si2=si)

    def is_si_exist(self, si_id: str) -> bool:
        with self.__mutex:
            return si_id in self.__si_dict

    def get_si(self) -> List[StrategyInsight]:
        with self.__mutex:
            return self.__si_dict[self.__si_list[-1]] # get latest si_id from self.__si_list

    def save_ci(self, ci: Dict[str, Any]) -> None:
        with self.__mutex:
            self.__ci = ci
            # self.__ci_list.append(ci)

    def get_ci(self) -> Dict[str, Any]:
        with self.__mutex:
            return self.__ci

    def save_ci_id(self, ci_id: str) -> None:
        with self.__mutex:
            self.__ci_id = ci_id

    def get_ci_id(self) -> str:
        with self.__mutex:
            return self.__ci_id

    def save_order(self, order: TS_Order) -> None:
        self.__order_cache.upsert_ele(key=order.platform_order_id, value=order)

    def save_orders(self, orders: List[TS_Order]) -> None:
        self.__order_cache.upsert_multi(keys=[order.platform_order_id for order in orders], values=orders)

    def __remove_old_orders(self
                            , order_list: List[TS_Order]
                            , order_dict: Dict[str, TS_Order]
                            , max_records: int) -> None:
        if len(order_dict) > max_records:
            # find old records
            extra_records = len(order_dict) - max_records
            old_record_count = 0
            old_idx_from_list = 0
            old_ids: List[str] = list()
            # find old order_id from order_list
            while old_record_count <= extra_records:
                for idx, old_record in enumerate(order_list):
                    if old_record.order_id == cur_order_id:
                        continue
                    else:
                        cur_order_id = old_record.order_id
                        old_record_count += 1
                        if old_record_count <= extra_records:
                            old_ids.append(cur_order_id)
                            old_idx_from_list = idx
                        break
            # remove records from dictionary
            for old_order_id in old_ids:
                if old_order_id in order_dict:
                    del order_dict[old_order_id]
            # remove records from list
            del order_list[:old_idx_from_list + 1]

    def save_trade(self, trade: TS_Trade) -> None:
        self.__trade_cache.upsert_ele(key=trade.trade_id, value=trade)

    def save_trades(self, trades: List[TS_Trade]) -> None:
        self.__trade_cache.upsert_multi(keys=[trade.trade_id for trade in trades], values=trades)

    def __remove_old_trades(self
                            , trade_list: List[TS_Trade]
                            , trade_dict: Dict[str, TS_Trade]
                            , max_records: int) -> None:
        # remove old records
        if len(trade_list) > max_records:
            extra_records = len(trade_list) - max_records
            # remove records from dictionary
            for old_record in trade_list[:extra_records]:
                if old_record.trade_id in trade_dict:
                    del trade_dict[old_record.trade_id]
            # remove records from list
            del trade_list[:extra_records]

    def is_trade_exist(self, trade_id: str) -> bool:
        return self.__trade_cache.is_exist(key=trade_id)

    def get_trade(self, trade_id: str) -> Union[bool, TS_Trade]:
        trade = self.__trade_cache.get_ele(key=trade_id)
        if None is trade:
            return (False, None)
        else:
            return (True, trade)

    def get_trades(self) -> List[TS_Trade]:
        return self.__trade_cache.get_records_in_time_series()

    def get_order(self, platform_order_id: str) -> Union[bool, TS_Order]:
        order = self.__order_cache.get_ele(key=platform_order_id)
        if None is order:
            return (False, None)
        else:
            return (True, order)

    def is_order_exist(self, platform_order_id: str) -> bool:
        return self.__order_cache.is_exist(key=platform_order_id)

    def get_orders(self) -> List[TS_Order]:
        return self.__order_cache.get_records_in_time_series()

    def save_fid(self, fid_num: int, fid_value: Any) -> None:
        bucket_num = int((fid_num - 1) / self.__bucket_size)
        if bucket_num not in self.__save_map:
            return None
        self.__save_map[bucket_num](fid_num=fid_num, fid_value=fid_value)

    def get_fid(self, fid_num: int) -> Union[bool, Any]:
        bucket_num = int((fid_num - 1) / self.__bucket_size)
        if bucket_num not in self.__get_map:
            return (False, None)

        return (True, self.__get_map[bucket_num](fid_num=fid_num))

    def __save_fid_int32(self, fid_num: int, fid_value: int) -> None:
        with self.__int_mutex:
            self.__int32_map[fid_num] = fid_value

    def __save_fid_int64(self, fid_num: int, fid_value: int) -> None:
        with self.__int_mutex:
            self.__int64_map[fid_num] = fid_value

    def __save_fid_float(self, fid_num: int, fid_value: float) -> None:
        with self.__float_mutex:
            self.__float_map[fid_num] = fid_value

    def __save_fid_double(self, fid_num: int, fid_value: float) -> None:
        with self.__float_mutex:
            self.__double_map[fid_num] = fid_value

    def __save_fid_string(self, fid_num: int, fid_value: str) -> None:
        with self.__str_mutex:
            self.__string_map[fid_num] = fid_value

    def __save_fid_bool(self, fid_num: int, fid_value: float) -> None:
        with self.__bool_mutex:
            self.__bool_map[fid_num] = fid_value

    def __save_fid_other(self, fid_num: int, fid_value: Any) -> None:
        with self.__other_mutex:
            if fid_num not in self.__save_other_map:
                return None

            self.__save_other_map[fid_num](fid_value)
            return None

    def __get_fid_int32(self, fid_num: int) -> Union[bool, int]:
        with self.__int_mutex:
            if fid_num in self.__int32_map:
                return (True, self.__int32_map[fid_num])
            else:
                return (False, None)

    def __get_fid_int64(self, fid_num: int) -> Union[bool, int]:
        with self.__int_mutex:
            if fid_num in self.__int64_map:
                return (True, self.__int64_map[fid_num])
            else:
                return (False, None)

    def __get_fid_float(self, fid_num: int) -> Union[bool, float]:
        with self.__float_mutex:
            if fid_num in self.__float_map:
                return (True, self.__float_map[fid_num])
            else:
                return (False, None)

    def __get_fid_double(self, fid_num: int) -> Union[bool, float]:
        with self.__float_mutex:
            if fid_num in self.__double_map:
                return (True, self.__double_map[fid_num])
            else:
                return (False, None)

    def __get_fid_string(self, fid_num: int) -> Union[bool, str]:
        with self.__str_mutex:
            if fid_num in self.__str_map:
                return (True, self.__str_map[fid_num])
            else:
                return (False, None)

    def __get_fid_bool(self, fid_num: int) -> Union[bool, bool]:
        with self.__bool_mutex:
            if fid_num in self.__bool_map:
                return (True, self.__bool_map[fid_num])
            else:
                return (False, None)

    def __get_fid_other(self, fid_num: int) -> Union[bool, Any]:
        with self.__other_mutex:
            if fid_num in self.__get_other_map:
                return (True, self.__get_other_map[fid_num]())
            else:
                return (False, None)

    def __is_same_si(self, si1: List[StrategyInsight], si2: List[StrategyInsight]) -> bool:
        if len(si1) != len(si2):
            return False

        si1_sorted = sorted(si1, key=lambda x: x.symbol_id)
        si2_sorted = sorted(si2, key=lambda x: x.symbol_id)
        for a, b in zip(si1_sorted, si2_sorted):
            if a != b:
                return False
        return True



import logging
import threading
from typing import List, Dict, Tuple, Union, Any

from sfdevtools.data_cache.DPage import DPage
from sfdevtools.data_cache.DStrategy import DStrategy
from sfdevtools.data_cache.DComponents import StrategyInsight, TS_Order, TS_Trade
import sfdevtools.grpc_protos.ts_cop_pb2 as ts_cop_pb2
import sfdevtools.grpc_protos.ts_cop_pb2_grpc as ts_cop_pb2_grpc
import sfdevtools.devTools.TimelyCache as TimelyCache

class DCache(object):
    def __init__(self):
        self.__logger: logging.Logger = None
        self.__pages: Dict[str, DPage] = None # key: QC Symbol ID, value: data page
        self.__strategy: DStrategy = DStrategy()
        self.__page_mutex: threading.Lock = threading.Lock()
        self.__order_snapshot_cache: TimelyCache.TimelyCache_DupKey\
            = TimelyCache.TimelyCache_DupKey() # key: platform_order_id, value: db_key
        self.__order_hist_cache: TimelyCache.TimelyCache_UniKey\
            = TimelyCache.TimelyCache_UniKey() # key: platform_order_id, value: TS_Order

    def init_component(self
                       , logger: logging.Logger
                       , max_hist_orders: int = 1000
                       , max_hist_trades: int = 1000
                       , max_hist_si: int = 1000) -> None:
        self.__logger = logger
        self.__strategy.init_component(logger=self.__logger
                                       , max_hist_orders=max_hist_orders
                                       , max_hist_si=max_hist_si
                                       , max_hist_trades=max_hist_trades)

    def get_strategy_insight(self) -> List[StrategyInsight]:
        return self.__strategy.get_strategy_insight()

    def save_si(self, si: List[StrategyInsight]) -> bool:
        return self.__strategy.save_si(si=si)

    def is_latest_si(self, si: List[StrategyInsight]) -> bool:
        return self.__strategy.is_latest_si(si=si)

    def is_si_exist(self, si_id: str) -> bool:
        return self.__strategy.is_si_exist(si_id=si_id)

    def get_ci_id(self) -> str:
        return self.__strategy.get_ci_id()

    def is_order_exist(self, platform_order_id: str) -> bool:
        return self.__strategy.is_order_exist(platform_order_id=platform_order_id)

    def get_order(self, platform_order_id: str) -> Union[bool, TS_Order]:
        return self.__strategy.get_order(platform_order_id=platform_order_id)

    def save_ci(self, ci: Dict[str, Any]) -> None:
        self.__strategy.save_ci(ci=ci)

    def save_ci_id(self, ci_id: str) -> None:
        self.__strategy.save_ci_id(ci_id=ci_id)

    def save_order(self, order: TS_Order) -> None:
        self.__strategy.save_order(order=order)

    def save_orders(self, orders: List[TS_Order]) -> None:
        self.__strategy.save_orders(orders=orders)

    def is_trade_exist(self, trade_id: str) -> bool:
        return self.__strategy.is_trade_exist(trade_id=trade_id)

    def get_trade(self, trade_id: str) -> Union[bool, TS_Trade]:
        return self.__strategy.get_trade(trade_id=trade_id)

    def save_trade(self, trade: TS_Trade) -> None:
        self.__strategy.save_trade(trade=trade)

    def save_trades(self, trades: List[TS_Trade]) -> None:
        self.__strategy.save_trades(trades=trades)

    def save_fid(self
                 , msg_id: str
                 , msg_type: ts_cop_pb2.Cop.MsgType
                 , fid_num: int
                 , fid_value: Any) -> None:

        # handle strategy level
        if msg_type == ts_cop_pb2.Cop.MsgType.Strategy:
            self.__strategy.save_fid(fid_num=fid_num, fid_value=fid_value)
            return

        # handle page level
        dpage = self.get_create_dpage(page_id=msg_id)
        dpage.save_fid(fid_num=fid_num, fid_value=fid_value)

        # handle extra strategy level
        if fid_num == ts_cop_pb2.FidNum.CI:
            for ci in fid_value:
                self.save_ci(ci=ci)
        elif fid_num == ts_cop_pb2.FidNum.SI:
            self.save_si(si=fid_value)
        elif fid_num == ts_cop_pb2.FidNum.Order:
            self.save_orders(orders=fid_value)
        elif fid_num == ts_cop_pb2.FidNum.Trade:
            self.save_trades(trades=fid_value)

    def get_fid(self
                , msg_id: str
                , msg_type: ts_cop_pb2.Cop.MsgType
                , fid_num: int) -> Union[bool, Union[bool, Any]]:
        # handle strategy level
        if msg_type == ts_cop_pb2.Cop.MsgType.Strategy:
            return self.__strategy.get_fid(fid_num=fid_num)

        # handle page level
        dpage = self.get_create_dpage(page_id=msg_id)
        return dpage.get_fid(fid_num=fid_num)

    def get_create_dpage(self, page_id: str) -> DPage:
        with self.__page_mutex:
            if page_id in self.__pages:
                return self.__pages[page_id]
            else:
                return self.__create_dpage(page_id=page_id)

    def get_order_snapshot_cache(self) -> TimelyCache.TimelyCache_DupKey:
        return self.__order_snapshot_cache

    def get_order_hist_cache(self) -> TimelyCache.TimelyCache_UniKey:
        return self.__order_hist_cache

    def __create_dpage(self, page_id: str) -> DPage:
        with self.__page_mutex:
            new_dpage = DPage()
            new_dpage.init_component(logger=self.__logger)
            self.__create_dpage[page_id] = new_dpage
            return new_dpage



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: peacock.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='peacock.proto',
  package='peacock',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\rpeacock.proto\x12\x07peacock\"1\n\x16SaveStrategyConfig_Msg\x12\x17\n\x0fstrategy_config\x18\x01 \x01(\t\"*\n\x18SaveStrategyConfig_Reply\x12\x0e\n\x06status\x18\x01 \x01(\t\">\n\x13SaveFileToColud_Msg\x12\x11\n\tfile_name\x18\x01 \x01(\t\x12\x14\n\x0c\x66ile_content\x18\x02 \x01(\t\"U\n\x15SaveRefDataFromQC_Msg\x12\x11\n\tfile_name\x18\x01 \x01(\t\x12\x14\n\x0c\x66ile_content\x18\x02 \x01(\t\x12\x13\n\x0b\x61\x63tion_mode\x18\x03 \x01(\t\"\x1a\n\tNoneReply\x12\r\n\x05reply\x18\x01 \x01(\t\"\x17\n\x04Ping\x12\x0f\n\x07message\x18\x01 \x01(\t\"\x17\n\x04Pong\x12\x0f\n\x07message\x18\x01 \x01(\t2\x9e\x02\n\x07Peacock\x12\x43\n\x0fSaveFileToColud\x12\x1c.peacock.SaveFileToColud_Msg\x1a\x12.peacock.NoneReply\x12G\n\x11SaveRefDataFromQC\x12\x1e.peacock.SaveRefDataFromQC_Msg\x1a\x12.peacock.NoneReply\x12X\n\x12SaveStrategyConfig\x12\x1f.peacock.SaveStrategyConfig_Msg\x1a!.peacock.SaveStrategyConfig_Reply\x12+\n\x0bHealthCheck\x12\r.peacock.Ping\x1a\r.peacock.Pongb\x06proto3'
)




_SAVESTRATEGYCONFIG_MSG = _descriptor.Descriptor(
  name='SaveStrategyConfig_Msg',
  full_name='peacock.SaveStrategyConfig_Msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='strategy_config', full_name='peacock.SaveStrategyConfig_Msg.strategy_config', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=26,
  serialized_end=75,
)


_SAVESTRATEGYCONFIG_REPLY = _descriptor.Descriptor(
  name='SaveStrategyConfig_Reply',
  full_name='peacock.SaveStrategyConfig_Reply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='peacock.SaveStrategyConfig_Reply.status', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=77,
  serialized_end=119,
)


_SAVEFILETOCOLUD_MSG = _descriptor.Descriptor(
  name='SaveFileToColud_Msg',
  full_name='peacock.SaveFileToColud_Msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='file_name', full_name='peacock.SaveFileToColud_Msg.file_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='file_content', full_name='peacock.SaveFileToColud_Msg.file_content', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=121,
  serialized_end=183,
)


_SAVEREFDATAFROMQC_MSG = _descriptor.Descriptor(
  name='SaveRefDataFromQC_Msg',
  full_name='peacock.SaveRefDataFromQC_Msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='file_name', full_name='peacock.SaveRefDataFromQC_Msg.file_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='file_content', full_name='peacock.SaveRefDataFromQC_Msg.file_content', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='action_mode', full_name='peacock.SaveRefDataFromQC_Msg.action_mode', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=185,
  serialized_end=270,
)


_NONEREPLY = _descriptor.Descriptor(
  name='NoneReply',
  full_name='peacock.NoneReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='reply', full_name='peacock.NoneReply.reply', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=272,
  serialized_end=298,
)


_PING = _descriptor.Descriptor(
  name='Ping',
  full_name='peacock.Ping',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='peacock.Ping.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=300,
  serialized_end=323,
)


_PONG = _descriptor.Descriptor(
  name='Pong',
  full_name='peacock.Pong',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='peacock.Pong.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=325,
  serialized_end=348,
)

DESCRIPTOR.message_types_by_name['SaveStrategyConfig_Msg'] = _SAVESTRATEGYCONFIG_MSG
DESCRIPTOR.message_types_by_name['SaveStrategyConfig_Reply'] = _SAVESTRATEGYCONFIG_REPLY
DESCRIPTOR.message_types_by_name['SaveFileToColud_Msg'] = _SAVEFILETOCOLUD_MSG
DESCRIPTOR.message_types_by_name['SaveRefDataFromQC_Msg'] = _SAVEREFDATAFROMQC_MSG
DESCRIPTOR.message_types_by_name['NoneReply'] = _NONEREPLY
DESCRIPTOR.message_types_by_name['Ping'] = _PING
DESCRIPTOR.message_types_by_name['Pong'] = _PONG
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SaveStrategyConfig_Msg = _reflection.GeneratedProtocolMessageType('SaveStrategyConfig_Msg', (_message.Message,), {
  'DESCRIPTOR' : _SAVESTRATEGYCONFIG_MSG,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.SaveStrategyConfig_Msg)
  })
_sym_db.RegisterMessage(SaveStrategyConfig_Msg)

SaveStrategyConfig_Reply = _reflection.GeneratedProtocolMessageType('SaveStrategyConfig_Reply', (_message.Message,), {
  'DESCRIPTOR' : _SAVESTRATEGYCONFIG_REPLY,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.SaveStrategyConfig_Reply)
  })
_sym_db.RegisterMessage(SaveStrategyConfig_Reply)

SaveFileToColud_Msg = _reflection.GeneratedProtocolMessageType('SaveFileToColud_Msg', (_message.Message,), {
  'DESCRIPTOR' : _SAVEFILETOCOLUD_MSG,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.SaveFileToColud_Msg)
  })
_sym_db.RegisterMessage(SaveFileToColud_Msg)

SaveRefDataFromQC_Msg = _reflection.GeneratedProtocolMessageType('SaveRefDataFromQC_Msg', (_message.Message,), {
  'DESCRIPTOR' : _SAVEREFDATAFROMQC_MSG,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.SaveRefDataFromQC_Msg)
  })
_sym_db.RegisterMessage(SaveRefDataFromQC_Msg)

NoneReply = _reflection.GeneratedProtocolMessageType('NoneReply', (_message.Message,), {
  'DESCRIPTOR' : _NONEREPLY,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.NoneReply)
  })
_sym_db.RegisterMessage(NoneReply)

Ping = _reflection.GeneratedProtocolMessageType('Ping', (_message.Message,), {
  'DESCRIPTOR' : _PING,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.Ping)
  })
_sym_db.RegisterMessage(Ping)

Pong = _reflection.GeneratedProtocolMessageType('Pong', (_message.Message,), {
  'DESCRIPTOR' : _PONG,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.Pong)
  })
_sym_db.RegisterMessage(Pong)



_PEACOCK = _descriptor.ServiceDescriptor(
  name='Peacock',
  full_name='peacock.Peacock',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=351,
  serialized_end=637,
  methods=[
  _descriptor.MethodDescriptor(
    name='SaveFileToColud',
    full_name='peacock.Peacock.SaveFileToColud',
    index=0,
    containing_service=None,
    input_type=_SAVEFILETOCOLUD_MSG,
    output_type=_NONEREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SaveRefDataFromQC',
    full_name='peacock.Peacock.SaveRefDataFromQC',
    index=1,
    containing_service=None,
    input_type=_SAVEREFDATAFROMQC_MSG,
    output_type=_NONEREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SaveStrategyConfig',
    full_name='peacock.Peacock.SaveStrategyConfig',
    index=2,
    containing_service=None,
    input_type=_SAVESTRATEGYCONFIG_MSG,
    output_type=_SAVESTRATEGYCONFIG_REPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='HealthCheck',
    full_name='peacock.Peacock.HealthCheck',
    index=3,
    containing_service=None,
    input_type=_PING,
    output_type=_PONG,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_PEACOCK)

DESCRIPTOR.services_by_name['Peacock'] = _PEACOCK

# @@protoc_insertion_point(module_scope)



import threading
from typing import List, Dict, Tuple, Any, Deque
from collections import deque

class MsgQ(object):
    def __init__(self, timeout: float = None):
        self.__q: Deque[Any] = deque()
        self.__mutex: threading.Lock = threading.Lock()
        self.__cond: threading.Condition = threading.Condition()
        self.__timeout: float = timeout

    def push(self, user_data: Any):
        with self.__mutex:
            self.__q.append(user_data)

        self.__cond.acquire()
        self.__cond.notify()
        self.__cond.release()

    def get(self) -> Any:
        self.__cond.acquire()
        self.__cond.wait(timeout=self.__timeout)
        self.__cond.release()

        with self.__mutex:
            if len(self.__q) == 0:
                return None
            else:
                return self.__q.popleft()

    def get_all(self) -> List[Any]:
        with self.__mutex:
            if len(self.__q) == 0:
                return None
            else:
                out_list = list(self.__q)
                self.__q = deque()
                return out_list

    def get_all_wait(self) -> List[Any]:
        self.__cond.acquire()
        self.__cond.wait(timeout=self.__timeout)
        self.__cond.release()

        with self.__mutex:
            if len(self.__q) == 0:
                return None
            else:
                out_list = list(self.__q)
                self.__q = deque()
                return out_list

    def size(self) -> int:
        with self.__mutex:
            return len(self.__q)



import time
from typing import List, Dict, Tuple, Any, Deque

class DiscontinueSeelp(object):
    def __init__(self
                 , sleep_time_ms: int
                 , d_sleep_time_ms: int = 100):
        self.__sleep_time_ms: int = sleep_time_ms
        self.__d_sleep_time_ms: int = d_sleep_time_ms

    def sleep(self, user_fun: Any) -> None:
        cur_sleep_time_ms: int = 0
        while cur_sleep_time_ms < self.__sleep_time_ms:
            next_sleep_time_ms = min(self.__sleep_time_ms - cur_sleep_time_ms, self.__d_sleep_time_ms)
            time.sleep(next_sleep_time_ms / 1000.0)
            cur_sleep_time_ms += next_sleep_time_ms

            if not user_fun():
                return



from typing import List, Dict, Tuple, Union, Any
import threading
import copy

class TimelyCache_DupKey(object):
    def __init__(self, max_size: int = 1000):
        self.__dict: Dict[Any, Any] = dict()
        self.__time_index: List[Union[Any, Any]] = list()
        self.__mutex: threading.Lock = threading.Lock()
        self.__max_size = max_size

    def upsert_multi(self, keys: List[Any], values: List[Any]) -> None:
        with self.__mutex:
            for key, value in zip(keys, values):
                self.__time_index.append((key, value))
                self.__dict[key] = value

            self.__remove_extra_records()

    def upsert_ele(self, key: Any, value: Any) -> None:
        with self.__mutex:
            self.__time_index.append((key, value))
            self.__dict[key] = value

            self.__remove_extra_records()

    def is_exist(self, key: Any) -> Any:
        with self.__mutex:
            return key in self.__dict

    def get_records_in_time_series(self) -> List[Any]:
        with self.__mutex:
            return [ele[1] for ele in self.__time_index]

    def get_ele(self, key: Any) -> Any:
        with self.__mutex:
            if key not in self.__dict:
                return None
            return copy.deepcopy(self.__dict[key])

    def size(self) -> int:
        with self.__mutex:
            return len(self.__dict)

    def items(self):
        with self.__mutex:
            for k, v in self.__dict.items():
                yield k, v

    def __remove_extra_records(self):
        if len(self.__dict) > self.__max_size:
            total_old_records = len(self.__dict) - self.__max_size
            # find the outdated key
            old_key, old_value = self.__time_index[0]
            last_n = 0
            old_record_count = 1
            old_key_list: List[Any] = list()
            # find old records from list
            for idx, ele in enumerate(self.__time_index):
                if not ele[0] == old_key:
                    old_record_count += 1
                    old_key_list.append(old_key)
                    old_key = ele[0]
                    if old_record_count > total_old_records:
                        last_n = idx
                        break
            # remove from dict
            for old_key in old_key_list:
                del self.__dict[old_key]
            # remove from list
            del self.__time_index[:last_n]

class TimelyCache_UniKey(object):
    def __init__(self, max_size: int = 1000):
        self.__dict: Dict[Any, Any] = dict()
        self.__time_index: List[Union[Any, Any]] = list()
        self.__mutex: threading.Lock = threading.Lock()
        self.__max_size = max_size

    def upsert_multi(self, keys: List[Any], values: List[Any]) -> None:
        with self.__mutex:
            for key, value in zip(keys, values):
                if key in self.__dict:
                    return

                self.__time_index.append((key, value))
                self.__dict[key] = value

            self.__remove_extra_records()

    def upsert_ele(self, key: Any, value: Any) -> None:
        with self.__mutex:
            if key in self.__dict:
                return

            self.__time_index.append((key, value))
            self.__dict[key] = value

            self.__remove_extra_records()

    def is_exist(self, key: Any) -> Any:
        with self.__mutex:
            return key in self.__dict

    def get_records_in_time_series(self) -> List[Any]:
        with self.__mutex:
            return [ele[1] for ele in self.__time_index]

    def get_ele(self, key: Any) -> Any:
        with self.__mutex:
            if key not in self.__dict:
                return None
            return copy.deepcopy(self.__dict[key])

    def size(self) -> int:
        with self.__mutex:
            return len(self.__dict)

    def items(self):
        with self.__mutex:
            for k, v in self.__dict.items():
                yield k, v

    def __remove_extra_records(self):
        if len(self.__dict) > self.__max_size:
            extra_n = len(self.__dict) - self.__max_size
            for ele in self.__time_index[:extra_n]:
                del self.__dict[ele[0]] # ele = (old_key, old_value)
            del self.__time_index[:extra_n]



import datetime

import sfdevtools.grpc_protos.ts_cop_pb2 as ts_cop_pb2
import sfdevtools.grpc_protos.ts_cop_pb2_grpc as ts_cop_pb2_grpc
from sfdevtools.data_cache.DComponents import StrategyInsight, TS_Order, TS_Trade

def conv_TS_Order_2_cop_order(order: TS_Order) -> ts_cop_pb2.TSOrder:
    ord_ele = ts_cop_pb2.TSOrder()

    ord_ele.symbol = order.symbol
    ord_ele.symbol_id = order.symbol_id
    ord_ele.quantity = order.quantity
    ord_ele.fill_quantity = order.fill_quantity
    ord_ele.parent_id = order.parent_id
    ord_ele.order_id = order.order_id
    ord_ele.platform_order_id = order.platform_order_id
    ord_ele.created = int(order.created.timestamp())
    ord_ele.last_update = int(order.last_update.timestamp())
    ord_ele.strategy_name = order.strategy_name
    ord_ele.live_mode = order.live_mode
    ord_ele.strategy_id = order.strategy_id
    ord_ele.order_status = order.order_status
    ord_ele.execution_broker = order.execution_broker
    ord_ele.clearing_broker = order.clearing_broker
    ord_ele.account = order.account
    ord_ele.price = order.price
    ord_ele.fee = order.fee
    ord_ele.fee_ccy = order.fee_ccy
    ord_ele.exchange = order.exchange
    ord_ele.ccy = order.ccy

    return ord_ele

def conv_TS_Trade_2_cop_trade(trd: TS_Trade) -> ts_cop_pb2.TSTrade:
    trd_ele = ts_cop_pb2.TSTrade()

    trd_ele.symbol = trd.symbol
    trd_ele.symbol_id = trd.symbol_id
    trd_ele.quantity = trd.quantity
    trd_ele.parent_id = trd.parent_id
    trd_ele.trade_id = trd.trade_id
    trd_ele.created = int(trd.created.timestamp())
    trd_ele.last_update = int(trd.last_update.timestamp())
    trd_ele.strategy_name = trd.strategy_name
    trd_ele.live_mode = trd.live_mode
    trd_ele.strategy_id = trd.strategy_id
    trd_ele.trade_status = trd.trade_status
    trd_ele.execution_broker = trd.execution_broker
    trd_ele.clearing_broker = trd.clearing_broker
    # trd_ele.settlement_date = trd.settlement_date # we don't have information from QC for now
    trd_ele.fx_ratio = trd.fx_ratio
    trd_ele.account = trd.account
    trd_ele.price = trd.price
    trd_ele.fee = trd.fee
    trd_ele.fee_ccy = trd.fee_ccy
    trd_ele.exchange = trd.exchange
    trd_ele.ccy = trd.ccy

    return trd_ele

def conv_SI_2_cop_si(si: StrategyInsight) -> ts_cop_pb2.SI:
    si_ele = ts_cop_pb2.SI()

    si_ele.symbol = si.symbol
    si_ele.symbol_id = si.symbol_id
    si_ele.ratio = si.ratio
    si_ele.parent_id = si.parent_id
    si_ele.si_id = si.si_id
    si_ele.created = int(si.created.timestamp())
    si_ele.last_update = int(si.last_update.timestamp())
    si_ele.strategy_name = si.strategy_name
    si_ele.live_mode = si.live_mode
    si_ele.strategy_id = si.strategy_id

    return si_ele

def conv_cop_order_2_TS_Order(ord_ele: ts_cop_pb2.TSOrder) -> TS_Order:
    order = TS_Order()

    order.symbol = ord_ele.symbol
    order.symbol_id = ord_ele.symbol_id
    order.quantity = ord_ele.quantity
    order.fill_quantity = ord_ele.fill_quantity
    order.parent_id = ord_ele.parent_id
    order.order_id = ord_ele.order_id
    order.platform_order_id = ord_ele.platform_order_id
    order.created = datetime.datetime.fromtimestamp(ord_ele.created)
    order.last_update = datetime.datetime.fromtimestamp(ord_ele.last_update)
    order.strategy_name = ord_ele.strategy_name
    order.live_mode = ord_ele.live_mode
    order.strategy_id = ord_ele.strategy_id
    order.order_status = ord_ele.order_status
    order.execution_broker = ord_ele.execution_broker
    order.clearing_broker = ord_ele.clearing_broker
    order.account = ord_ele.account
    order.price = ord_ele.price
    order.fee = ord_ele.fee
    order.fee_ccy = ord_ele.fee_ccy
    order.exchange = ord_ele.exchange
    order.ccy = ord_ele.ccy

    return order

def conv_cop_trade_2_TS_Trade(trd_ele: ts_cop_pb2.TSTrade) -> TS_Trade:
    trd = TS_Trade()

    trd.symbol = trd_ele.symbol
    trd.symbol_id = trd_ele.symbol_id
    trd.quantity = trd_ele.quantity
    trd.parent_id = trd_ele.parent_id
    trd.trade_id = trd_ele.trade_id
    trd.created = datetime.datetime.fromtimestamp(trd_ele.created)
    trd.last_update = datetime.datetime.fromtimestamp(trd_ele.last_update)
    trd.strategy_name = trd_ele.strategy_name
    trd.live_mode = trd_ele.live_mode
    trd.strategy_id = trd_ele.strategy_id
    trd.trade_status = trd_ele.trade_status
    trd.execution_broker = trd_ele.execution_broker
    trd.clearing_broker = trd_ele.clearing_broker
    trd.fx_ratio = trd_ele.fx_ratio
    trd.account = trd_ele.account
    trd.price = trd_ele.price
    trd.fee = trd_ele.fee
    trd.fee_ccy = trd_ele.fee_ccy
    trd.exchange = trd_ele.exchange
    trd.ccy = trd_ele.ccy

    return trd

def conv_cop_si_2_SI(si_ele: ts_cop_pb2.SI) -> StrategyInsight:
    si = StrategyInsight(symbol=si_ele.symbol, ratio=si_ele.ratio)

    si.symbol = si_ele.symbol
    si.symbol_id = si_ele.symbol_id
    si.ratio = si_ele.ratio
    si.parent_id = si_ele.parent_id
    si.si_id = si_ele.si_id
    si.created = datetime.datetime.fromtimestamp(si_ele.created / 1000)
    si.last_update = datetime.datetime.fromtimestamp(si_ele.last_update / 1000)
    si.strategy_name = si_ele.strategy_name
    si.live_mode = si_ele.live_mode
    si.strategy_id = si_ele.strategy_id

    return si



from typing import List, Dict, Tuple, Union, Any
import threading
import copy

class SafeDict(object):
    def __init__(self):
        self.__dict: Dict[Any, Any] = dict()
        self.__mutex: threading.Lock = threading.Lock()

    def upsert_ele(self, key: Any, value: Any) -> None:
        with self.__mutex:
            self.__dict[key] = value

    def get_ele(self, key: Any) -> Any:
        with self.__mutex:
            if key not in self.__dict:
                return None
            return copy.deepcopy(self.__dict[key])

    def size(self) -> int:
        with self.__mutex:
            return len(self.__dict)

    def items(self):
        with self.__mutex:
            for k, v in self.__dict.items():
                yield k, v
